/*
 * $Header: /cvs/distapps/openmap/src/openmap/com/bbn/openmap/proj/RhumbCalculator.java,v 1.1.2.4 2005/08/11 21:03:20 dietrick Exp $
 *
 * Copyright 2004 OBR Centrum Techniki Morskiej, All rights reserved.
 *
 */

package com.bbn.openmap.proj;

import java.lang.Math;
import com.bbn.openmap.LatLonPoint;

/**
 * 
 * @version $Header:
 *          /cvs/distapps/openmap/src/openmap/com/bbn/openmap/proj/RhumbCalculator.java,v
 *          1.1 2004/05/10 20:53:58 dietrick Exp $
 * @author pawkub
 * 
 * Klasa zawiera metody pozwalajce przeprowadza obliczenia zwizane
 * z tzw. Rhumblinem. Klasa zawiera kilka oglnych wzorw i poza
 * OpenMap nie jest z niczym zwizana. Mona j wrzuci do OpenMapy i
 * uy zawartych w niej wzorw tam gdzie to bdzie potrzebne (a jest
 * takich miejsc kilka).
 */
public class RhumbCalculator {

    /**
     * Konstruktor jest prywatny, bo (na razie) klasa zawiera tylko
     * kilka metod statycznych.
     */
    private RhumbCalculator() {}

    /*
     * Metoda pozwala pbliczy pooenie punktu na Rhumb Line, znajc
     * pooenie punktu odniesienia, azymut i odlego. @param point
     * Punkt odniesienia @param azimuth azymut w stopniach @oparam
     * dist odlego w metrach @return punkt (obiekt klasy
     * LatLonPoint)
     */
    public static LatLonPoint calculatePointOnRhumbLine(LatLonPoint point,
                                                        float azimuth,
                                                        float dist) {
        double lat1 = (double) point.radlat_;
        double lon1 = (double) point.radlon_;
        double d = (double) dist / 1855.3 * Math.PI / 10800.0;
        double lat = 0.0;
        double lon = 0.0;
        lat = lat1 + d * Math.cos(azimuth);
        double dphi = Math.log((1 + Math.sin(lat)) / Math.cos(lat))
                - Math.log((1 + Math.sin(lat1)) / Math.cos(lat1));
        double dlon = 0.0;
        if (Math.abs(Math.cos(azimuth)) > Math.sqrt(0.00000000000001)) {
            dlon = dphi * Math.tan(azimuth);
        } else { // along parallel
            dlon = Math.sin(azimuth) * d / Math.cos(lat1);
        }
        lon = mod(lon1 - dlon + Math.PI, 2 * Math.PI) - Math.PI;
        //System.out.println("calculatePointOnRhumbLine: lat1 =
        // "+lat1+"+ lon1 = "+lon1 + " lat = "+lat+"+ lon = "+lon);
        return new LatLonPoint((float) lat, (float) lon, true);
    }

    /**
     * Metoda pozwala znormowa warto do podanego zakresu
     * 
     * @param y normaowana warto
     * @param x zakres
     * @return normowana warto
     */
    private static double mod(double y, double x) {
        double ret;
        if (y >= 0) {
            ret = y - x * (int) (y / x);
        } else {
            ret = y + x * ((int) (-y / x) + 1);
        }
        return ret;
    }

    /**
     * Metoda pozwala obliczy odlego pomidzy punktami liczon po
     * rhumbline.
     * 
     * @param p1 punkt geograficzny
     * @param p2 punkt geograficzny
     * @return odlego
     */
    public static float getDistanceBetweenPoints(LatLonPoint p1, LatLonPoint p2) {
        double lat1 = (double) p1.radlat_;
        double lon1 = (double) p1.radlon_;
        double lat2 = (double) p2.radlat_;
        double lon2 = (double) p2.radlon_;
        double d = 0.0;
        double tc = 0.0;
        double dlon_W = mod(lon2 - lon1, 2 * Math.PI);
        double dlon_E = mod(lon1 - lon2, 2 * Math.PI);
//        double dphi = Math.log((1 + Math.sin(lat2)) / Math.cos(lat2))
//                - Math.log((1 + Math.sin(lat1)) / Math.cos(lat1));
        tc = getAzimuthBetweenPoints(p1, p2);
        if (Math.abs(lat1 - lat2) < Math.sqrt(0.00000000000001)) {
            d = Math.min(dlon_W, dlon_E) * Math.cos(lat1); // distance
                                                           // along
                                                           // parallel
        } else {
            d = Math.abs((lat2 - lat1) / Math.cos(tc));
        }
        float dist = (float) (d * 10800.0 / Math.PI * 1855.3);
        //System.out.println("DWrong = " + distWrong + " DOK = " +
        // dist);
        return dist;
    }

    /**
     * Metoda pozwala obliczy azymut pomidzy punktami (namiar od
     * pierwszego do drugiego)
     * 
     * @param p1 pierwszy punkt geograficzny
     * @param p2 drugi punkt geograficzny
     * @return namiar w radianach
     */
    public static float getAzimuthBetweenPoints(LatLonPoint p1, LatLonPoint p2) {
        double lat1 = (double) p1.radlat_;
        double lon1 = (double) p1.radlon_;
        double lat2 = (double) p2.radlat_;
        double lon2 = (double) p2.radlon_;

        double tc = 0.0;
        double dlon_W = mod(lon2 - lon1, 2 * Math.PI);
        double dlon_E = mod(lon1 - lon2, 2 * Math.PI);
        double dphi = Math.log((1 + Math.sin(lat2)) / Math.cos(lat2))
                - Math.log((1 + Math.sin(lat1)) / Math.cos(lat1));
        if (dlon_W < dlon_E) {// West is the shortest
            tc = mod(Math.atan2(-dlon_W, dphi), 2 * Math.PI);
        } else {
            tc = mod(Math.atan2(dlon_E, dphi), 2 * Math.PI);
        }
        return (float) tc;
    }

}